/* 
* File: nvalsrnd.c
* By: Isaac Michaud and David Hiebeler
*     http://www.math.umaine.edu/~hiebeler
*
* This program demonstrates a problem with the random number 
* generator "random()", which goes away when using the Mersenne Twister Prime
* available from: http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/MT2002/emt19937ar.html
* You will need to download mt19937ar.c and mt19937ar.h (from the tar-file at the above location)
* to compile this source code.  The "randproblem.c" file will compile as-is without any other
* dependencies.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include "mt19937ar.h"


int main() {
  struct timeval tp;
  struct timezone *tzp;
  int i, iters=10000000, left=0, right=0;
  // int i,test;
  double r;

  /* Use the current time in milliseconds to seed the random
   * number generator
   */
  tzp = NULL;
  gettimeofday(&tp, tzp);
  init_genrand((unsigned int) tp.tv_usec);

  for(i=0;i<iters;i++){
    genrand_real2();
    r = genrand_real2();
    if (r < 0.25) {
      /* 25% of the time, we make one extra call to genrand_real2() */
      genrand_real2();
      left++;
    } else {
      /* the other 75% of the time, we make three extra calls to genrand_real2() */
      genrand_real2();
      genrand_real2();
      genrand_real2();
      right++;
    }
    if (i % 100000 == 0) {
      printf("Left: %d  Right: %d  R/L Ratio %f (should be 3)\n",left,right,(double)right/left);
    } 
  }  
  return 0;
}
